searchData={"items":[{"type":"module","title":"Medea.Formatter","doc":"A formatter for JSON logs.","ref":"Medea.Formatter.html"},{"type":"function","title":"Medea.Formatter.format/4","doc":"Format messages into structured JSON logs.","ref":"Medea.Formatter.html#format/4"},{"type":"type","title":"Medea.Formatter.time/0","doc":"","ref":"Medea.Formatter.html#t:time/0"},{"type":"module","title":"Medea.Translator","doc":"Translates structured log \"reports\" into encoded JSON iodata.","ref":"Medea.Translator.html"},{"type":"extras","title":"Medea","doc":"# Medea\n\nSorceress, wife to Jason, tiny logger of nicely formatted structured messages.\n\nMedea is a tiny package that provides a [formatter][form] and a\n[translator][tran] to preserve structured logs. For example, here is the\ndifference between structured logging with inspection and the structured JSON\nwe'd expect:\n\n```\n# Inspected\n\"message\":\"%{args: %{id: 1, on: false}}\"\n\n# Structured\n\"message\":{\"args\":{\"id\":1, \"on\":false}}\n```\n\nThe former contains an inspected Elixir map, while the latter is easily parsable\nas JSON.\n\nWith Medea, all terms are safely escaped and converted to JSON printable values.\nThat means structured logging like this:\n\n```elixir\nLogger.info(event: %{name: :stuff, safe: false}, user: %User{id: 123})\n```\n\nOutputs nested JSON like this:\n\n```json\n{\n  \"level\":\"info\",\n  \"message\":{\n    \"event\":{\n      \"name\":\"stuff\",\n      \"safe\":false\n    },\n    \"user\":{\n      \"id\":123\n    }\n  },\n  \"metadata\":[],\n  \"time\":\"2022-10-11T15:07:33.000\"\n}\n```","ref":"readme.html"},{"type":"extras","title":"Installation - Medea","doc":"The package can be installed by adding `medea` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:medea, \"~> 0.2\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Medea","doc":"Medea has two components: `Medea.Formatter` and `Medea.Translator`. For complete\nfunctionality, both components must be configured.\n\nFirst, configure the default formatter:\n\n```elixir\nconfig :logger, :default_formatter,\n  format: {Medea.Formatter, :format},\n  metadata: [:request_id]\n```\n\nNext, enable the translator at the top of your `c:Application.start/2` function:\n\n```elixir\ndef start(_type, _args) do\n  Logger.add_translator({Medea.Translator, :translate})\n\n  children = [\n    # whichever children you have\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Logging With Ecto - Medea","doc":"If you're using Ecto, you should disable the default logger:\n\n```elixir\nconfig :my_app, MyApp.Repo, log: false\n```\n\nNow replace it with a simple [telemetry](https://hexdocs.pm/telemetry/readme.html)-based handler that logs structured\nqueries with metadata:\n\n```elixir\ndefmodule MyApp.EctoLogger do\n  require Logger\n\n  def handle_event(_event, measure, %{query: query, repo: repo}, level) do\n    query_meta = Map.take(measure, ~w(idle_time queue_time query_time decode_time total_time)a)\n\n    Logger.log(level, query, repo: inspect(repo), query: query_meta)\n  end\nend\n```\n\nAnd attach:\n\n```elixir\n:telemetry.attach(\n  \"ecto-logger\",\n  [:my_app, :repo, :query],\n  &MyApp.EctoLogger.handle_event/4,\n  :info\n)\n```","ref":"readme.html#logging-with-ecto"},{"type":"extras","title":"Custom logger formatting - Medea","doc":"If you need to format certain values in the log set by other libraries,\nfor example, `:otel_span_id` and `:otel_trace_id`, then you can configure\nmedea to send certain keypaths to your own implementation.\n\nThis configuration relies on compile-time-only configuration for efficiency,\nso it cannot be set in `runtime.exs`.\n\nFor example:\n\nBefore:\n\n```json\n{\n  \"level\":\"info\",\n  \"message\":{\"foo\":\"bar\"},\n  \"metadata\":{\n    \"otel_span_id\":[0,1,2,3,4],\n    \"otel_trace_id\":[3,4,5]\n  },\n  \"time\":\"2022-10-11T15:07:33.000\"\n}\n```\n\n```elixir\nconfig :medea,\n  formatters: %{\n    [:metadata, :otel_span_id] => {MyApp.Logs, :format, []},\n    [:metadata, :otel_trace_id] => {MyApp.Logs, :format, []}\n  }\n```\n\n```elixir\ndefmodule MyApp.Logs do\n  def format(_keypath, charlist) when is_list(charlist) do\n    to_string(charlist)\n  end\nend\n```\n\nAfter:\n\n```json\n{\n  \"level\":\"info\",\n  \"message\":{\"foo\":\"bar\"},\n  \"metadata\":{\n    \"otel_span_id\":\"abc123\",\n    \"otel_trace_id\":\"def456\"\n  },\n  \"time\":\"2022-10-11T15:07:33.000\"\n}\n```","ref":"readme.html#custom-logger-formatting"},{"type":"extras","title":"Custom Jason.Encoder Implementations - Medea","doc":"Structs that implement [Jason.Encoder](https://hexdocs.pm/jason/Jason.Encoder.html) will use that protocol.\nIf any implementation is undesirable, as is the case with `Ecto.Association.NotLoaded`\nand `Ecto.Schema.Metadata` which both raise errors as of `3.12.3`, it can be disabled at runtime.\n\n```elixir\nconfig :medea, except: [Ecto.Association.NotLoaded, Ecto.Schema.Metadata]\n```\n\nAdditional documentation can be found at [https://hexdocs.pm/medea](https://hexdocs.pm/medea).\n\n[form]: https://hexdocs.pm/logger/Logger.Formatter.html\n[tran]: https://hexdocs.pm/logger/Logger.Translator.html","ref":"readme.html#custom-jason-encoder-implementations"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.3.0 — 2025-05-29 - Changelog","doc":"* Add escape-hatch for custom key value formatting\n\n  OpenTelemetry libraries add some metadata to logger, but they add them as charlists\n  (`otel_span_id` and `otel_trace_id`) which ends up as a list of integers in the JSON log. For\n  log correlation with some services (such as Datadog), they need these IDs to be in hex format,\n  which means I need to be able to hook into the log formatting for these particular keys and\n  values and format them.\n\n  This adds a compile-config-based escape hatch for formatting values in the logger message.","ref":"changelog.html#v0-3-0-2025-05-29"},{"type":"extras","title":"v0.2.0 — 2024-09-23 - Changelog","doc":"* Handle formatting improper lists\n\n  Improper lists can't be handled by standard iterators such as `Enum.map/2`, which made it\n  impossible to convert them into a standard, loggable format.\n\n* Check for implementation of `Jason.Encoder` for structs\n\n  If a struct has a custom implementation of `Jason.Encoder` we'll clean the internals but put\n  everything back where we found it so the custom implementation could redact fields, for example.","ref":"changelog.html#v0-2-0-2024-09-23"},{"type":"extras","title":"v0.1.2 — 2022-01-12 - Changelog","doc":"* Inspect non-atom/string keys\n\n  Instead of exploding on map keys in Jason if they were a list like `[:foo, \"bar\"]` or a map,\n  etc., we'll allow strings or atoms as before, but `inspect` otherwise.","ref":"changelog.html#v0-1-2-2022-01-12"},{"type":"extras","title":"v0.1.1 — 2022-12-06 - Changelog","doc":"* Ensure full keyword list before encoding as map\n\n  This fixes situations where there are erlang-style option lists with both atoms and tuples, e.g.\n  `[{Mod, []}, :enabled]`","ref":"changelog.html#v0-1-1-2022-12-06"},{"type":"extras","title":"v0.1.0 — 2022-10-10 - Changelog","doc":"Initial release with a translator and formatter for structured JSON logs.","ref":"changelog.html#v0-1-0-2022-10-10"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}